@@ -1,5 +1,5 @@
 package Ryu::Source;
-$Ryu::Source::VERSION = '0.013';
+$Ryu::Source::VERSION = '0.014';
 use strict;
 use warnings;
 
@@ -151,8 +151,33 @@ It is expected that the interface and internals of this method will vary greatly
 sub from {
     my $class = shift;
     my $src = (ref $class) ? $class : $class->new;
-    if(my $ref = ref($_[0])) {
-        if($ref eq 'GLOB') {
+    if(my $from_class = blessed($_[0])) {
+        if($from_class->isa('Future')) {
+            retain_future(
+                $_[0]->on_ready(sub {
+                    my ($f) = @_;
+                    if($f->failure) {
+                        $src->fail($f->from_future);
+                    } elsif(!$f->is_cancelled) {
+                        $src->finish;
+                    } else {
+                        $src->emit($f->get);
+                        $src->finish;
+                    }
+                })
+            );
+            return $src;
+        } else {
+            die 'Unknown class ' . $from_class . ', cannot turn it into a source';
+        }
+    } elsif(my $ref = ref($_[0])) {
+        if($ref eq 'ARRAY') {
+            $src->{on_get} = sub {
+                $src->emit($_) for @{$_[0]};
+                $src->finish;
+            };
+            return $src;
+        } elsif($ref eq 'GLOB') {
             if(my $fh = *{$_[0]}{IO}) {
                 my $code = sub {
                     while(read $fh, my $buf, 4096) {
@@ -462,6 +487,18 @@ sub suffix {
     }, $src);
 }
 
+sub sprintf_methods {
+    my ($self, $fmt, @methods) = @_;
+    my $src = $self->chained(label => (caller 0)[3] =~ /::([^:]+)$/);
+    $self->completed->on_ready(sub {
+        shift->on_ready($src->completed) unless $src->completed->is_ready
+    });
+    $self->each_while_source(sub {
+        my ($item) = @_;
+        $src->emit(sprintf $fmt, map $item->$_, @methods)
+    }, $src);
+}
+
 =head2 as_list
 
 Resolves to a list consisting of all items emitted by this source.
